import { JSX as JSX_2 } from 'react';
import { LoggerBasic } from 'shared-react/plugins/logger-basic.model';
import { NodeOptions } from 'shared-react/adaptors/editor-adaptor.model';
import { noteNodeName } from 'shared-react/nodes/scripture/usj/NoteNode';
import { OnClick } from 'shared-react/nodes/scripture/usj/NoteNode';
import { default as React_2 } from 'react';
import { ScriptureReference } from 'papi-components';
import { Usj } from 'shared/converters/usj/usj.model';
import { usxStringToJson } from 'shared/converters/usj/usx-to-usj';

/**
 * Scripture Editor for USJ. Created for use in Platform.Bible.
 * @see https://github.com/usfm-bible/tcdocs/blob/usj/grammar/usj.js
 *
 * @param props.usj - USJ Scripture data.
 * @param props.viewModeState - View Mode state object containing the viewMode and the function to
 *   set it.
 * @param props.scrRefState - Scripture reference state object containing the ref and the function
 *   to set it.
 * @param props.nodeOptions - Options for each node.
 * @param props.nodeOptions[].noteCallers - Possible note callers to use when caller is
 *   '+' for NoteNode.
 * @param props.nodeOptions[].onClick - Click handler for NoteNode.
 * @param props.isReadonly - Is the editor readonly or editable (default).
 * @param props.logger - Logger instance.
 * @returns the editor element.
 */
export declare function Editor<TLogger extends LoggerBasic>({ usj, viewOptionsState, scrRefState, nodeOptions, isReadonly, logger, }: EditorProps<TLogger>): JSX_2.Element;

declare type EditorProps<TLogger extends LoggerBasic> = {
    /** Scripture data in USJ form */
    usj?: Usj;
    /** View Options state */
    viewOptionsState?: [
    viewOptions: ViewOptions | undefined,
    setViewOptions: React_2.Dispatch<React_2.SetStateAction<ViewOptions>>
    ];
    /** Scripture Ref state */
    scrRefState?: [
    scrRef: ScriptureReference,
    setScrRef: React_2.Dispatch<React_2.SetStateAction<ScriptureReference>>
    ];
    /** Options for each node */
    nodeOptions?: UsjNodeOptions;
    /** Is the editor readonly or editable */
    isReadonly?: boolean;
    /** Logger instance */
    logger?: TLogger;
};

/**
 * Get view option properties based on the view mode.
 * @param viewMode - View mode of the editor.
 * @returns the view options if the view is defined, `undefined` otherwise.
 */
export declare function getViewOptions(viewMode: string | undefined): ViewOptions | undefined;

export { Usj }

declare interface UsjNodeOptions extends NodeOptions {
    [noteNodeName]?: {
        noteCallers?: string[];
        onClick?: OnClick;
    };
}

export { usxStringToJson }

export declare type ViewMode = ViewNameKey;

declare const viewModeToViewNames: {
    formatted: string;
    unformatted: string;
};

declare type ViewNameKey = keyof typeof viewModeToViewNames;

export declare type ViewOptions = {
    /** USFM markers are visible, editable or hidden */
    markerMode: "visible" | "editable" | "hidden";
    /** is the text indented */
    isIndented: boolean;
    /** is the text in a plain font */
    isPlainFont: boolean;
};

/**
 * Convert view options to view mode if the view exists.
 * @param viewOptions - View options of the editor.
 * @returns the view mode if the view is defined, `undefined` otherwise.
 */
export declare function viewOptionsToMode(viewOptions: ViewOptions | undefined): ViewMode | undefined;

export { }
